# include <stdio.h>
# include <stdlib.h>
# include <math.h>
# include <string.h>
# include <vector>
using namespace std;
typedef vector<double> Data;

double f(Data &x)
{
    return x[0]*x[0]+x[1]*x[1]-cos(18*x[0])-cos(18*x[1]);
}

Data gradient(Data &x)
{
    Data g;
    g.resize(x.size());
    g[0]=2.0*x[0]+18.0*sin(18.0*x[0]);
    g[1]=2.0*x[1]+18.0*sin(18.0*x[1]);
    return g;
}

double fl(Data &x,Data &s,double l)
{
    Data x1;
    x1.resize(x.size());
    int i;
    for(i=0;i<x1.size();i++)
        x1[i]=x[i]+l * s[i];
    return f(x1);
}

void goldenSearch(Data &x,Data &s,double &a,double &b)
{
    double phi = (sqrt(5.0)-1.0)/2.0;
    int iteration = 1;
    while(true)
    {
        double x1 = a+(1-phi)*(b-a);
        double x2 = a+phi * (b-a);
        double diff =fabs(x1-x2);
        if(diff<1e-6) break;
        if(fl(x,s,x1)<fl(x,s,x2))
        {
            b = x2;
        }
        else
        {
            a=x1;
        }
        iteration++;
        printf("Iteration=%4d Range=[%10.5lg,%10.5lg]\n",
               iteration,a,b);
    }
}

int main()
{
    double a=1.0;
    double b=10.0;
    int i;
    Data x,s;
    x.resize(2);
    x[0]=0.02;
    x[1]=0.02;
    s.resize(2);
    do{
        a = 1.0;
        b = 10.0;
        s = gradient(x);
        for(i=0;i<s.size();i++)
        {
            s[i]=-s[i];
        }
        goldenSearch(x,s,a,b);
        double diff = 0.0;
        for(i=0;i<s.size();i++)
        {
            double xold = x[i];
            x[i]=x[i]+a*s[i];
            diff+=(xold - x[i]) * (xold - x[i]);
        }
        printf("Diff = %20.10lg F(x)=%lf\n",diff,f(x));
        if(diff<1e-6) break;
    }while(true);
    return 0;
}
